/*-------------------------------------------------------------------------
File name   : dlxMisc.e
Title       : CPU Run Time Environment
Project     : DLX Validation
Developer   : Prabhat Mishra 
Created     : 08-28-2003
---------------------------------------------------------------------------
Copyright 2004 (c) Regents of University of California, Irvine
-------------------------------------------------------------------------*/

<'
struct cell {
	address: word;
	value: word;
};

extend global { 
    setup_test() is also {
        set_config(print, radix, hex);
        set_config(cover, mode, on_interactive, show_mode, both);
        set_config(print, items, 100);
        set_config(run, tick_max, 900000, exit_on, command);
        set_config(gen, max_structs, 55000);
    };

    readReg(r: cpu_reg_t, R: list of word, busyR: list of bool, busy: *bool) : word is 
	{
		var indx: int = r.as_a(int);
		if (busyR[indx]) then {
            busy = TRUE;
            return -1;
        } else {
            busy = FALSE;
            return R[indx];
        };
	};

    getType(opcode : cpu_opcode_t) : inst_t is {
    	case opcode  {
			[LB..SNEI]: { 
            	return I_type; 
            }; 
      		[ADD..NEF, NOP]: { 
            	return R_type; 
            }; 
      		[J, JAL, TRAP, RFE]: { 
            	return J_type; 
            };  
      		default: { 
            	out (" ** ERROR: Wrong opcode **");
            };
    	};  
  	};

	print_instr(instr: cpu_instr_s, pc : word, unit: string) is {
		var iinst : I_type cpu_instr_s;
		var rinst : R_type cpu_instr_s;
		var jinst : J_type cpu_instr_s;

		outf("DUT %s instr : \t", unit);

		if (instr.opcode == NOP) then {
			out(instr.opcode); // NOP
//				out("    DUT PC: ", pc);
		} else if (instr.instType == I_type) then {
			iinst = instr.as_a(I_type cpu_instr_s);
			out(append(instr.opcode, "\t", iinst.dest, ", ", iinst.src1, ", #", iinst.src2));
			out("    DUT PC: ", pc, ",  dest: ", instr.result, ",  src1: ", instr.src1Val, ",  src2: ", instr.src2Val);
		} else if (instr.instType == R_type) then {
			rinst = instr.as_a(R_type cpu_instr_s);
			out(append(instr.opcode, "\t", rinst.dest, ", ", rinst.src1, ", ", rinst.src2));
			out("    DUT PC: ", pc, ",  dest: ", instr.result, ",  src1: ", instr.src1Val, ",  src2: ", instr.src2Val);
		} else if (instr.instType == J_type) then {
			jinst = instr.as_a(J_type cpu_instr_s);
			out(append(instr.opcode, "\t", "#", jinst.src1));
			out("    DUT PC: ", pc, " src1: ", instr.src1Val);
		} else {
			out(" ** ERROR **");
		};
	}; 

	getInstClass(opcode: cpu_opcode_t) : int is {
        case opcode {
            [ADDF, SUBF, LTF, GTF, LEF, GEF, EQF, NEF]: {
                result = 1; // Float operations
            };
            [MULT, MULTF]: {
                result = 2; // Multiply operations
            };
            [DIV, DIVF]: {
                result = 3; // Division operations
            };
            default: {
                result = 0; // Integer operations
            };
        };
    }; 
};
'>
