/*-------------------------------------------------------------------------
File name   : dlxTest.e
Title       : Functional Test
Project     : DLX Validation
Developer   : Prabhat Mishra
Created     : 08-28-2003
Description : Generate test cases
            :
---------------------------------------------------------------------------
Copyright 2004 (c) Regents of University of California, Irvine
-------------------------------------------------------------------------*/

<'
import dlxTop;

extend cpu_instr_s {

// Uncomment to do constrained test generation
	// R0 is always 0 and only readable
	// R31 is modified by many Branch/Jmp instructions - skip it
//	when R_type cpu_instr_s {
//		keep dest != R0;
//	};
//	when I_type cpu_instr_s {
//		keep dest != R0;
//	};

//    keep soft instType == select {
//        50: I_type;     
//        44: R_type;
//         6: J_type;
//    };

//	when R_type cpu_instr_s {keep opcode in [ADD];};
//	when I_type cpu_instr_s {keep opcode in [SB];};
};

extend sys { 
    // generate specified number of instructions
	keep instrs.size() == 11;

    post_generate() is also { 
        gen instrs;        // start generating stream of instructions 
    }; 
};

define <rinst'action>
  "<name> <dest'exp> <src1'exp> <src2'exp> config" as {
    var inst : cpu_instr_s = new;
    var type : inst_t;
    inst.opcode = <name>;
		
	type = getType(inst.opcode);
	inst.instType = type;

	case type {
		[R_type]: {
			var rInst: R_type cpu_instr_s;
			rInst = inst.as_a(R_type cpu_instr_s);

			rInst.dest = <dest'exp>;
			rInst.src1 = <src1'exp>;
			rInst.src2 = <src2'exp>;
		};
		default: {
			out(" **** ERROR: ", inst.opcode, " is not valid R_type operation ****"); 
		};
	};
	sys.instrs.add(inst);
};

define <branch'action>
  "<name> <src1'exp> <src2'num> config" as {
    var inst : cpu_instr_s = new;
    var type : inst_t;
    inst.opcode = <name>;
   
    type = getType(inst.opcode);
    inst.instType = type;

    case type {
        [I_type]: { // Example: BEQZ R5 0x777, dest unused
            var iInst: I_type cpu_instr_s;
            iInst = inst.as_a(I_type cpu_instr_s);

            iInst.src1 = <src1'exp>;
            iInst.src2 = <src2'num>;
        };
        default: {
            out(" **** ERROR: ", inst.opcode, " is not valid I_type operation ****");
        };
    };
    sys.instrs.add(inst);
};

define <jmpr'action>
  "<name> <src1'exp> conf" as {
    var inst : cpu_instr_s = new;
    var type : inst_t;
    inst.opcode = <name>;
   
    type = getType(inst.opcode);
    inst.instType = type;

    case type {
        [I_type]: { // Example: BEQZ R5 0x777, dest unused
            var iInst: I_type cpu_instr_s;
            iInst = inst.as_a(I_type cpu_instr_s);

            iInst.src1 = <src1'exp>;
        };
        default: {
            out(" **** ERROR: ", inst.opcode, " is not valid I_type operation ****");
        };
    };
    sys.instrs.add(inst);
};
define <iinst'action>
  "<name> <dest'exp> <src1'exp> <src2'num> config" as {
    var inst : cpu_instr_s = new;
    var type : inst_t;
    inst.opcode = <name>;

    type = getType(inst.opcode);
    inst.instType = type;

    case type {
        [I_type]: {
            var iInst: I_type cpu_instr_s;
            iInst = inst.as_a(I_type cpu_instr_s);

            iInst.dest = <dest'exp>;
            iInst.src1 = <src1'exp>;
            iInst.src2 = <src2'num>;
        };
        default: {
            out(" **** ERROR: ", inst.opcode, " is not valid I_type operation ****");
        };
    };
    sys.instrs.add(inst);
};

define <jinst'action>
  "<name> <src1'num> config" as {
    var inst : cpu_instr_s = new;
    var type : inst_t;
    inst.opcode = <name>;

    type = getType(inst.opcode);
    inst.instType = type;

    case type {
        [J_type]: {
            var jInst: J_type cpu_instr_s;
            jInst = inst.as_a(J_type cpu_instr_s);

            jInst.src1 = <src1'num>;
        };
        default: {
            out(" **** ERROR: ", inst.opcode, " is not valid J_type operation ****");
        };
    };
    sys.instrs.add(inst);
};

define <nop'action>
  "NOP" as {
    var inst : cpu_instr_s = new;
    var type : inst_t;
    inst.opcode = NOP;

    sys.instrs.add(inst);
};

// Add any directed tests that you want to run alongwith the tests
// generated by Specman Elite. I wrote the rules above to read these
// tests. The way rules are written, for every test (instruction), add
// the word "config" at the end. Try modifying the rules to get rid of
// the "config" word.

extend sys {
    run() is also {
      J 0x444 config;
		// R2 = 2
//        ADDI R2 R0 0x2 config;
		// R5 = FFFF
        ADDI R5 R0 0xFFFF config;
		// R6 = 10001
//        ADD R6 R5 R2 config;
		// R7 = FFFFFFFF
//		MULT R7 R5 R6 config;

		// Overflow
//		MULT R8 R7 R2 config;
//		ADDF R9 R7 R7 config;

		// Branch misprediction
//        BEQZ R11 0x777 config;

		// Page Fault
		LH R12 R15 0xFFFF config;
		NOP;
		NOP;
		NOP;
		NOP;
		NOP;
		NOP;
		NOP;
		NOP;
     // JR R7 conf;
    };
};
'>
